import requests
from flask.wrappers import Response
from requests.adapters import HTTPAdapter
from urllib3.util import Retry
import json

headers = {
    'Content-Type': 'application/json'
    }
session = requests.Session()
retry = Retry(connect=5,backoff_factor=0.2)
adapter = HTTPAdapter(max_retries=retry)
session.mount('http://', adapter)
session.mount('https://', adapter)

def get(url:str, apiKey:str,env_url_arg:str='',payload:dict={}):
    local_url = url.format(apiKey,env_url_arg)
    if payload == {}:
        return session.get(local_url, headers=headers)
        #return requests.request("GET", )
    else:
        return session.get(local_url, headers=headers, data=json.dumps(payload))
        #return requests.request("GET", local_url, headers=headers, data=json.dumps(payload))

def post(url:str, apiKey:str,keyword:str,keyword_detail:str,reference:str,method:str,value:str,message:str,env_url_arg:str=''):
    local_url = url.format(apiKey,env_url_arg)
    payload = {  
        "keyword": keyword,
        "keyword_detail": keyword_detail,
        "reference": reference,
        "method":method,
        "message": message,
        "value":value
    }

    #return requests.request("POST", local_url, headers=headers, data=json.dumps(payload))
    return session.post(local_url, headers=headers, data=json.dumps(payload))

def put(url:str, apiKey:str,env_url_arg:str='',payload:dict={}):
    local_url = url.format(apiKey,env_url_arg)
    if payload == {}:
        #return requests.request("PUT", local_url, headers=headers)
        return session.put(local_url, headers=headers)
    else:
        #return requests.request("PUT", local_url, headers=headers, data=json.dumps(payload))
        return session.put(local_url, headers=headers, data=json.dumps(payload))

def delete(url:str, apiKey:str,payload:dict,env_url_arg:str=''):
    local_url = url.format(apiKey,env_url_arg)
    if payload != {}:
        return session.delete(local_url, headers=headers, data=json.dumps(payload))
    return Response(status=400)



import json
import sys
import random
import requests
from requests.structures import CaseInsensitiveDict

def SlackPostCreatePair(urlCreatePair, newTokenAddr, hcResult, tokenName, tokenSymbol):

    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"

    msgTokenA = 'New Token: <https://etherscan.io/address/{0}|{1}>'.format(newTokenAddr,newTokenAddr)
    honeypotLink = 'Link <https://honeypot.is/ethereum.html?address={0}|Honeypot.is>'.format(newTokenAddr)
    dexLink = 'Link <https://www.dextools.io/app/ether/pair-explorer/{0}|DexTools.io'.format(newTokenAddr)

    slack_data = {
        "blocks": 
        [
            {"type": "header","text": {"type": "plain_text","text": "New Pair created - {0} ({1})".format(tokenName, tokenSymbol),"emoji": True}},
            {"type": "section","fields": [{"type": "mrkdwn","text": msgTokenA,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": honeypotLink,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": dexLink,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": str(hcResult),}]}
        ]
    }

    data=json.dumps(slack_data)
    resp = requests.post(urlCreatePair, headers=headers, data=data)

def SlackPostNewContract(url, tokenName:str, tokenSymbol:str, address:str, txHash:str, blockNr:str, tokenClass:str, hcResult):

    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"

    #if tokenClass == "ERC20":
    #    resp = requests.post(url, headers=headers, data=data)
    #    print(resp.status_code)

    txHashLink = 'TX Hash: <https://etherscan.io/tx/{0}|{1}>'.format(txHash,txHash)
    addressLink = 'Address: <https://etherscan.io/address/{0}|{1}>'.format(address,address)
    honeypotLink = 'Link <https://honeypot.is/ethereum.html?address={0}|Honeypot.is>'.format(address)
    dexLink = 'Link <https://www.dextools.io/app/ether/pair-explorer/{0}|DexTools.io'.format(address)
    slack_data = {
        #"username": "NotificationBot",
        #"icon_emoji": ":satellite:",
        #"channel" : "#somerandomcahnnel",
        "blocks": [
            {"type": "header","text": {"type": "plain_text","text": "New Contract - {0} ({1})".format(tokenName, tokenSymbol),"emoji": True}},
            {"type": "section","fields": [{"type": "mrkdwn","text": txHashLink,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": addressLink,}]},
            {"type": "section","fields": [{"type": "plain_text","text": 'Block: {0}'.format(blockNr),}]},
            {"type": "section","fields": [{"type": "plain_text","text": 'Token: {0}'.format(tokenClass),}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": honeypotLink,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": dexLink,}]},
            {"type": "section","fields": [{"type": "mrkdwn","text": str(hcResult),}]}
        ]
    }

    data=json.dumps(slack_data)

    if tokenClass == "ERC20" or tokenClass == "ERC20 (not verified)":
        resp = requests.post(url, headers=headers, data=data)

def SlackPostLiquidity(url, methodName:str, tokenAddr:str, txHash:str):

    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"

    msgAddr = 'Token: <https://etherscan.io/address/{0}|{1}>'.format(tokenAddr,tokenAddr)
    msgHash = 'Tx Hash: <https://etherscan.io/tx/{0}|{1}>'.format(txHash,txHash)
    
    slack_data = {
        "blocks": 
        [
            {"type": "header","text": {"type": "plain_text","text": "Rug Alert - {0}".format(methodName),"emoji": True}},
            {"type": "section","fields": [{"type": "mrkdwn", "text": msgAddr,}]},
            {"type": "section","fields": [{"type": "mrkdwn", "text": msgHash,}]}
        ]
    }

    data=json.dumps(slack_data)
    resp = requests.post(url, headers=headers, data=data)


import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry
import json
from web3 import Web3
from cfc_package.neblmask_api import tokenObjects,payloadBuilder

headers = {
    'Content-Type': 'application/json'
    }

session = requests.Session()
retry = Retry(connect=5,backoff_factor=0.2)
adapter = HTTPAdapter(max_retries=retry)
session.mount('http://', adapter)
session.mount('https://', adapter)
def get(url:str, apiKey:str,env_url_arg:str='',payload:dict={}):
    local_url = url.format(apiKey,env_url_arg)
    if payload == {}:
        return session.get(local_url, headers=headers)
        #return requests.request("GET", )
    else:
        return session.get(local_url, headers=headers, data=json.dumps(payload))
        #return requests.request("GET", local_url, headers=headers, data=json.dumps(payload))

# def post(apiKey:str,token_address:str,liquidity_pool_address:str,buyAmount:int,is_active:str,
#     is_completed:str,chain_id:int,env_url_arg:str=''):
#     local_url = url.format(apiKey,env_url_arg)
#     payload = {  
#         "token_address": token_address,
#         "liquidity_pool_address":liquidity_pool_address,
#         "buy_amount":buyAmount,
#         "is_active":is_active,
#         "is_completed":is_completed,
#         "chain_id":chain_id,
#     }
#     return requests.request("POST", local_url, headers=headers, data=json.dumps(payload))

def put(url:str, apiKey:str,env_url_arg:str='',payload:dict={}):
    local_url = url.format(apiKey,env_url_arg)
    if payload == {}:
        return session.put(local_url, headers=headers)
    else:
        return session.put(local_url, headers=headers, data=json.dumps(payload))
  

def updateCodeCheck(tokenAddress:str,neblMaskApiKey:str,env_url_arg:str):
    response = tokenObjects.get_codecheck(apiKey=neblMaskApiKey,
    token_address=Web3.toChecksumAddress(tokenAddress),
    env_url_arg=env_url_arg)
        
    result = json.loads(response.text)
    if len(result) != 0:
        check_result = 2
        if int(result[0]['is_edited_y']) == int(result[0]['is_malicious_n']) and int(result[0]['is_edited_n']) == 0:
            check_result = 1
        if int(result[0]['is_malicious_y']) > 0:
            check_result = 3
        payload = payloadBuilder.put([['contract_address',Web3.toChecksumAddress(tokenAddress),'eq']],[['code_check_result',str(check_result)]])
        put(apiKey=neblMaskApiKey,env_url_arg=env_url_arg,payload=payload)


from web3 import Web3

###################################################################
# Global variables ################################################
###################################################################


# Ankr
url_ankr_eth = 'https://{0}@apis-sj.ankr.com/{1}/eth/fast/main'
url_ankr_bsc = 'https://{0}@apis-sj.ankr.com/{1}/binance/full/main'
url_ankr_ftm = 'https://{0}@apis-sj.ankr.com/{1}/fantom/full/main'
url_ankr_avax = 'https://{0}@apis-sj.ankr.com/{1}/avax/archive/main'
url_ankr_matic = 'https://{0}@apis-sj.ankr.com/{1}/polygon/full/main'

# other provider endpoints
url_infura_rinkeby='https://rinkeby.infura.io/v3/{0}'
url_infura_eth_mainnet='https://mainnet.infura.io/v3/{0}'
url_binance_bsc_testnet='https://data-seed-prebsc-1-s1.binance.org:8545/'
url_binance_bsc_mainnet='https://bsc-dataseed.binance.org/'
url_defibit_bsc_mainnet='https://bsc-dataseed1.defibit.io/'
url_ninicoin_bsc_mainnet='https://bsc-dataseed1.ninicoin.io/'
url_getblock_bsc_mainnet='https://bsc.getblock.io/mainnet/?api_key={0}'
url_getblock_bsc_testnet='https://bsc.getblock.io/testnet/?api_key={0}'
url_getblock_eth_mainnet='https://eth.getblock.io/mainnet/?api_key={0}' 

# Addresses of interest
AddrUniSwapV2FactoryAdress = '0x5C69bEe701ef814a2B6a3EDD4B1652CB9cc5aA6f'
unicryptContractAddr = '0x663A5C229c09b049E36dCc11a9B0d4a8Eb9db214'
AddrLP_uniswapV2 = '0x7a250d5630B4cF539739dF2C5dAcb4c659F2488D'
AddrLP_uniswapV3 = '0xe592427a0aece92de3edee1f18e0157c05861564'
AddrLP_sushiSwap = '0xd9e1ce17f2641f24ae83637ab66a2cca9c378b9f'
AddrPancakeSwap = '0x10ED43C718714eb63d5aA57B78B54704E256024E'
AddrWETH = '0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2'
AddrWBNB = '0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c'

def get_address_weth_or_wbnb(chain_id, w3:Web3):
    if chain_id == 1:
        return w3.toChecksumAddress(AddrWETH)
    elif chain_id == 56:
        return w3.toChecksumAddress(AddrWBNB)
    return None

def get_address_swap_router(chain_id, w3:Web3):
    if chain_id == 1:
        return w3.toChecksumAddress(AddrLP_uniswapV2)
    elif chain_id == 56:
        return w3.toChecksumAddress(AddrPancakeSwap)
    return None

def get_address_uniswapV3_router(chain_id, w3:Web3):
    if chain_id == 1:
        return w3.toChecksumAddress(AddrLP_uniswapV3)
    return None